
//Include la libreria HCSR04
#include <HCSR04.h>

//Include la libreria Wire
#include <Wire.h>

//Include la libreria Oled SSD1306
#include <ACROBOTIC_SSD1306.h>

//Variabili per salvare la distanza dell'oggetto, distanza, e la misura iniziale: distanza0
float distanza, distanza0;

//Variabile per salvare la stringa da mostrare sullo schermo Oled
char misura[10];

//Se la differenza tra distanza e distanza0 è maggiore della soglia errore
//la nuova distanza e un messaggio di avvertimento è mostrato sullo schermo
const float errore=20.0;

//Inizializza il sensore ad ultrasuoni
//XIAO D1-> HCSR04 Trig
//XIAO D2-> HCSR04 Echo
UltraSonicDistanceSensor distanceSensor(1, 2);  


//Mostra i messaggi sull'Oled
void displayMessage()
{
  
  //Imposta la posizione del cursore, linea 4 carattere 0
  oled.setTextXY(4,0);  
   
 //Mostra la misura         
  oled.putString(misura);   

  //Imposta la posizione del cursore, linea 6 carattere 0    
  oled.setTextXY(6,0);   

  //Mostra il messaggio di Avvertimento        
  oled.putString("OGG. RILEVATO"); 
  
}

void setup()
{
  

  //Inizializza la porta seriale per mandare messaggi
  Serial.begin(9600);
  
  //Inizializza la libreria Wire e si collega al bus I2C come master o slave
  Wire.begin();  

  //Inizializza lo schermo Oled SSD1306
  oled.init();      

  //Ripulisce lo schermo               
  oled.clearDisplay();     

  //Imposta la posizione del cursore alla linea 0 carattere 0     
  oled.setTextXY(0,0);              

  //Mostra un messaggio di Intestazione
  oled.putString("RILEVAMENTO");  
  
  //Imposta il cursore alla linea 1 carattere 0  
  oled.setTextXY(1,0);       

  //Mostra un messaggio di Intestazione      
  oled.putString("OGGETTI");     

  //All'avvio misura la distanza in cm senza l'intrusione di oggetti
  distanza0=distanceSensor.measureDistanceCm();

  //Ripulisce il contenuto di misura
  memset(misura,'\0',sizeof(misura));
  
  //Invia i dati da mostrare alla stringa misura
  sprintf(misura, "D in cm: %.2f", distanza0);
  
  //Imposta il cursore alla linea 4 carattere 0   
  oled.setTextXY(4,0);  
   
  //Mostra la misura sull'Oled      
  oled.putString(misura);         
}

void loop()
{

  
  //Misura la distanza in cm nel campo di visione del sensore ultrasonico
  distanza=distanceSensor.measureDistanceCm();

  
  //Invia la misura sulla porta seriale Arduino
  Serial.println(distanza);
  
  //Ripulisce in contenuto di misura
  memset(misura,'\0',sizeof(misura));


   //Invia i dati da mostrare alla stringa misura 
  sprintf(misura, "D in cm: %.2f", distanza);

  //Aspetta 550 millisecondi
   delay(500);


   //Se la nuova misura differisce da quella iniziale di un quantita superiore a quanto definito nella
   //variabile errore, la nuova misura è mostrata sullo schermo Oled con un messaggio di avvertimento
   if(abs(distanza-distanza0)>errore)
   {

    //Mostra il contenuto di misura sull'Oled
     displayMessage();
   }
  
}
